/*******************************************************************************
 MRF24WG SPI Stub Functions

  Summary: Functions to control MRF24WG RESET and HIBERNATE pins need by the
           Universal Driver.  Functions in this module should not be called
           directly by the application code.

  Description: This module is specific to:
                * PIC32MX795F512L on the Explorer16
                * MRF24WG PICTail plugged into the PICTail Plus connector towards
                   on the pin 1 edge with module facing the MCU (SPI Slot 1)
*******************************************************************************/

/* MRF24WG0M Universal Driver
*
* Copyright (c) 2012-2014, Microchip Technology, Inc. <www.microchip.com>
* Contact Microchip for the latest version.
*
* This program is free software; distributed under the terms of BSD
* license:
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* 1.    Redistributions of source code must retain the above copyright notice, this
*        list of conditions and the following disclaimer.
* 2.    Redistributions in binary form must reproduce the above copyright notice,
*        this list of conditions and the following disclaimer in the documentation
*        and/or other materials provided with the distribution.
* 3.    Neither the name(s) of the above-listed copyright holder(s) nor the names
*        of its contributors may be used to endorse or promote products derived
*        from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
* DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
* OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
* OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//==============================================================================
//                                  INCLUDES
//==============================================================================
// board-specific includes
#include "wf_universal_driver.h"
#include "stm32l4xx_hal.h"

//==============================================================================
//                                  DEFINES
//==============================================================================

//==============================================================================
//                                  LOCAL VARIABLES
//==============================================================================
bool g_intDisabled = false;
SPI_HandleTypeDef spiHandle;
//==============================================================================
//                                  LOCAL FUNCTIONS
//==============================================================================


/*****************************************************************************
  Function:
    void WF_SpiInit(void);

  Summary:
    Initializes SPI controller used to communicate with MRF24WG

  Description:
    Called by Universal Driver during initialization to initialize SPI interface.

 Parameters:
    None

  Returns:
    None

  Remarks:
    Should not be called directly from application code.
*****************************************************************************/
void WF_SpiInit(void)
{
  __HAL_RCC_GPIOA_CLK_ENABLE();
  __HAL_RCC_GPIOB_CLK_ENABLE();
  __HAL_RCC_GPIOE_CLK_ENABLE();
  __HAL_RCC_SPI1_CLK_ENABLE();

  GPIO_InitTypeDef GPIO_InitStruct;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
  GPIO_InitStruct.Pin = GPIO_PIN_0;
  HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
  HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);

  // Initialize the SPI pins
  GPIO_InitStruct.Mode = GPIO_MODE_AF_PP;
  GPIO_InitStruct.Alternate = GPIO_AF5_SPI1;
  GPIO_InitStruct.Pin = GPIO_PIN_1 | GPIO_PIN_7;
  HAL_GPIO_Init(GPIOA, &GPIO_InitStruct);

  GPIO_InitStruct.Pin = GPIO_PIN_14;
  HAL_GPIO_Init(GPIOE, &GPIO_InitStruct);

  // Configure the SPI connected to the Pmod module.
  spiHandle.Instance = SPI1;
  spiHandle.Init.Mode = SPI_MODE_MASTER;
  spiHandle.Init.Direction = SPI_DIRECTION_2LINES;
  spiHandle.Init.DataSize = SPI_DATASIZE_8BIT;
  spiHandle.Init.CLKPolarity = SPI_POLARITY_LOW;
  spiHandle.Init.CLKPhase = SPI_PHASE_1EDGE;
  spiHandle.Init.NSS = SPI_NSS_SOFT;
  spiHandle.Init.BaudRatePrescaler = SPI_BAUDRATEPRESCALER_16;
  spiHandle.Init.FirstBit = SPI_FIRSTBIT_MSB;
  spiHandle.Init.TIMode = SPI_TIMODE_DISABLE;
  spiHandle.Init.CRCCalculation = SPI_CRCCALCULATION_DISABLE;
  spiHandle.Init.NSSPMode = SPI_NSS_PULSE_DISABLE;

  HAL_SPI_Init(&spiHandle);
}

/*****************************************************************************
  Function:
    void WF_SpiEnableChipSelect(void);

  Summary:
    Selects the MRF24WG SPI by setting the CS line low.

  Description:
    Called by Universal Driver when preparing to transmit data to the MRF24WG.
    Note that MRF24WG interrupt must be disabled before MRF24WG SPI chip select
    is selected.

 Parameters:
    None

  Returns:
    None

  Remarks:
    Should not be called directly from application code.
*****************************************************************************/
void WF_SpiEnableChipSelect(void)
{
   //Save current interrupt state
   g_intDisabled = WF_isEintDisabled();
   //Disable interrupts
   WF_EintDisable();

   //Assert CS line
   //usleep(1);
   HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_RESET);
   //usleep(1);
}

/*****************************************************************************
  Function:
    void WF_SpiDisableChipSelect(void);

  Summary:
    Deselects the MRF24WG SPI by setting CS high.

  Description:
    Called by Universal Driver after completing an SPI transaction.  Note that MRF24WG
    interrupt state must be restored after SPI chip select is deselected.

 Parameters:
    None

  Returns:
    None

  Remarks:
    Should not be called directly from application code.
*****************************************************************************/
void WF_SpiDisableChipSelect(void)
{
   //Deassert CS line
   //usleep(1);
   HAL_GPIO_WritePin(GPIOB, GPIO_PIN_0, GPIO_PIN_SET);
   //usleep(1);

   //Restore interrupt state
   if(!g_intDisabled)
   {
      //Enable interrupts
      WF_EintEnable();
   }
}


/*****************************************************************************
  Function:
    void WF_SpiTxRx(uint8_t *p_txBuf, uint16_t txLength, uint8_t *p_rxBuf, uint16_t rxLength);

  Summary:
    Transmits and receives SPI bytes with the MRF24WG.

  Description:
    Called by Universal Driver to communicate with the MRF24WG.

 Parameters:
    p_txBuf  -- pointer to the transmit buffer
    txLength -- number of bytes to be transmitted from p_txBuf
    p_rxBuf  -- pointer to receive buffer
    rxLength -- number of bytes to read and copy into p_rxBuf

  Returns:
    None

  Remarks:
    Should not be called directly from application code.
*****************************************************************************/
void WF_SpiTxRx(uint8_t *p_txBuf,
                uint16_t txLength,
                uint8_t *p_rxBuf,
                uint16_t rxLength)
{
   uint16_t i;
   uint16_t byteCount;
   uint8_t dataRx;//[128];
   uint8_t dataTx;//[128];

   //Total number of bytes to transfer
   if(txLength >= rxLength)
      byteCount = txLength;
   else
      byteCount = rxLength;

   //

   //printf("TX: ");
   //Transfer data
   for(i = 0; i < byteCount; i++)
   {
      //Get the character to be written
      if(i < txLength)
	dataTx = p_txBuf[i];
      else
	dataTx = 0xFF;
      //printf("%d ", dataTx[i]);
      //dataRx[i] = 0;
      HAL_SPI_TransmitReceive(&spiHandle, &dataTx, &dataRx, 1, 100);

      if(i < rxLength)
	p_rxBuf[i] = dataRx;
    }
     //printf("\r\n");
    //Write character
    //HAL_SPI_TransmitReceive(&spiHandle, dataTx, dataRx, byteCount, 100);

    //for(i = 0; i < rxLength; i++)
    //  p_rxBuf[i] = dataRx[i];
}
